@@ -1,6 +1,11 @@
-/*Author: Yu*/
+/*Author: Yu, Zhaoxia*/
 /*Date: 2007/04/02*/
 
+/* Modified: Jason Sinnwell to do permutations until p.threshold met, 
+   as implemented in haplo.score permutations */
+/* Date: 2008/9/22 */
+
+
 #include<stdio.h>
 #include<stdlib.h>
 #include<string.h>
@@ -9,9 +14,9 @@
 /*ranAS183_seed reads three random seeds*/
 static int ranAS183_seed(int, int, int);
 /*ranAS183 generates random values*/
-static double ranAS183();
+static double ranAS183(void);
 /*redefine haplotype memberships for a subset of SNPs from a set of SNPs*/
-static int creatsubhap();
+static int creatsubhap(void);
 /*calculate the chi^2 statistics for a 2-by-h table*/
 static double chisq2h(int *);
 /*calculate a Mantel-Haenszel statistic*/
@@ -34,11 +39,11 @@ static int **haplist;
 static double *post, *pos; /*posterior probabilities and SNPs' physical positions*/
 static int *newhap1codesingle, *newhap2codesingle;
 
-static double r2_threshold, mh_threshold, haplo_freq_min; /*parameters*/
+static double r2_threshold, mh_threshold, haplo_freq_min, p_threshold; /*parameters*/
 static int nsnp, nhap, nsub, npost, newnhap, flag, hap_df;
 
 
-void seqhap(
+void seqhapC(
 	    int *nsnp_c, /*number of SNPs*/
 	    int *nsub_c, /*number of subjects*/
 	    int *npost_c, /*length of posterior probabilities*/
@@ -54,6 +59,10 @@ void seqhap(
 
 	    int *seed_c, /*random seeds*/
 	    int *nperm_c, /*number of permutations to calculate P-values*/
+	    int *min_perm_c, /* min number permutations in Besag/Clifford approach */
+	    int *max_perm_c, /* max number permutations in Besag/Clifford approach */
+	    double *p_threshold_c, /* perm p-value threshold */
+
 	    double *lamda_c, /*threshold for the MH test*/
 	    double *r2_threshold_c, /*threshold to ignore one marker when two are in high r^2*/
 	    double *haplo_freq_min_c, /*the minimum haplotype frequency that should used*/
@@ -79,23 +88,30 @@ void seqhap(
   nsnp = *nsnp_c;
   r2_threshold = *r2_threshold_c;
   mh_threshold = *lamda_c;
+  p_threshold = *p_threshold_c;
   int N_PERM = *nperm_c;
+  int MIN_PERM = *min_perm_c;  
+  int MAX_PERM = *max_perm_c; 
   nsub = *nsub_c;
   npost = *npost_c;
   nhap = *nhap_c;
   haplo_freq_min = *haplo_freq_min_c;
 
-  FILE *fp;
-  int nperm, si, i, j, k;
+  int si, i, j, k;
   double chi_stat0i[nsnp], chi_max=0, chi_max0=0, chi_pi[nsnp], chi_p=0,
     hap_stat0i[nsnp], hap_min=1, hap_min0=1, hap_pi[nsnp], hap_p=0,
     sum_stat0i[nsnp], sum_min=1, sum_min0=1, sum_pi[nsnp], sum_p=0; 
+
   for(i=0; i<nsnp; i++){
-    chi_pi[i]=0; hap_pi[i]=0; sum_pi[i]=0;}
+    chi_pi[i]=0; hap_pi[i]=0; sum_pi[i]=0;
+  }
 
   int seed[3];
+
   for(i=0; i<3; i++) seed[i]=seed_c[i];
-  double ran1();
+
+  /* double ran1(); */
+
   ranAS183_seed(seed[0],seed[1],seed[2]);
 
   haplist = (int **) malloc (nhap *sizeof (int *));
@@ -137,7 +153,7 @@ void seqhap(
   j=0;
 
   /*calculate statistics using the original disease status*/
-  printf("\n");
+
   for(si=0; si<nsnp; si++)
     {
       inlist_length=1; inlist[inlist_length-1]=si;
@@ -152,14 +168,7 @@ void seqhap(
 	}/*search for the list of SNPs to be combined to SNP si and update the summary statistic*/
       if(inlist_length==1) {hap_stat0i[si]=chi_stat0i[si]; hap_df=1;}
       else hap_stat0i[si] = chisq2h(disease); /*update the haplotype statistic*/
-      /*
-      printf("snp%d\n",si+1);
-      printf("single-locus method:  chi-square %lf, d.f. %d\n", chi_stat0i[si],1);
-      printf("Sequential method combines snps ");
-      for(i=0; i<inlist_length; i++) printf("%d ",inlist[i]+1); printf("\n");
-      printf("Seq-haplotype method: chi-square %lf, d.f. %d\n", hap_stat0i[si],hap_df);
-      printf("Seq-summary method:   chi-square %lf, d.f. %d\n\n", sum_stat0i[si],inlist_length);
-      */
+ 
 
       /*save for Splus*/
       for(i=0; i<inlist_length; i++) {inlist_c[j]=inlist[i]+1;j++;}
@@ -197,15 +206,19 @@ void seqhap(
 	nondup_disease[j] = disease[i];
 	j ++;}
 
-  double chi_tmp, hap_tmp, sum_tmp;
-  for(nperm=0; nperm<N_PERM; nperm++)
-    {
+  double chi_tmp, hap_tmp, sum_tmp, h_region;
+  int doneperm=0;            /* jps */
+  N_PERM = 0;
+  while(!doneperm) {  /* while() used to keep permuting until p-threshold met -JPS*/
+
       /*permute disease status*/
       for(i=0; i<nsub; i++)
 	perm_rand[i]=ranAS183();
       permute(nondup_disease, perm_rand, perm_disease);
       chi_max=0; hap_min=1; sum_min=1;
 
+      N_PERM++; /*jps*/
+
       /*calculate the three statistics for a permutated data set*/
       for(si=0; si<nsnp; si++)
 	{
@@ -227,25 +240,39 @@ void seqhap(
 	  hap_tmp = pchisq(doublek/2, hap_tmp/2);
 
 	  /*global*/
-	  if(chi_tmp>chi_max) chi_max=chi_tmp;
-	  if(hap_tmp<hap_min) hap_min=hap_tmp;
-	  if(sum_tmp<sum_min) sum_min=sum_tmp;	  
-
+	  if(chi_tmp > chi_max) chi_max=chi_tmp;
+	  if(hap_tmp < hap_min) hap_min=hap_tmp;
+	  if(sum_tmp < sum_min) sum_min=sum_tmp;	  
+ 
 	  /*pointwise*/
-	  if(chi_tmp>chi_stat0i[si]) chi_pi[si]++;
-	  if(hap_tmp<hap_stat0i[si]) hap_pi[si]++;
-	  if(sum_tmp<sum_stat0i[si]) sum_pi[si]++;
+	  if(chi_tmp > chi_stat0i[si]) chi_pi[si]++;
+	  if(hap_tmp < hap_stat0i[si]) hap_pi[si]++;
+	  if(sum_tmp < sum_stat0i[si]) sum_pi[si]++;
 	}
-      /*global*/
-      if(chi_max>chi_max0) chi_p++;
-      if(hap_min<hap_min0) hap_p++;
-      if(sum_min<sum_min0) sum_p++;
-    }
 
-  /*global P-vlaues based on permutation*/
-  *chi_p_region_c=chi_p/N_PERM;
-  *hap_p_region_c=hap_p/N_PERM;
-  *sum_p_region_c=sum_p/N_PERM;
+      /*global*/
+      if(chi_max > chi_max0) chi_p++;
+      if(hap_min < hap_min0) hap_p++;
+      if(sum_min < sum_min0) sum_p++;
+
+      /* }  old for() stop  */
+   
+      if( N_PERM >= MIN_PERM ) {  
+      /* added by JPS:  apply Besag and Clifford permutation 
+         p-values rules to region p-values */
+	h_region = 1/((p_threshold * p_threshold) + 1/N_PERM);
+	if( ((h_region <= hap_p) & (h_region <= chi_p) & (h_region <= sum_p)) | (N_PERM == MAX_PERM) ) {
+ 	  doneperm = 1;
+	}
+      }
+  }
+
+  /*global P-values based on permutation*/
+  *chi_p_region_c = chi_p/N_PERM;
+  *hap_p_region_c = hap_p/N_PERM;
+  *sum_p_region_c = sum_p/N_PERM;
+
+  *nperm_c = N_PERM;
 
   /*pointwisw P-vlaues based on permutation*/
   for(si=0; si<nsnp; si++)
@@ -264,142 +291,131 @@ void seqhap(
 }
 
 /*search for the list of SNPs to be combined to SNP si*/
-static double combine(int *d, int si, int k)
-{
+static double combine(int *d, int si, int k) {
   int j, flagl, flagr;
   double mh_sum=0, mh_tmp;
-	  if(si-k>=0 && si+k<nsnp)
-	    if(pos[si]-pos[si-k] > pos[si+k]-pos[si])
-	      {
-		/*check si+k*/
-		flagr=1;
-		if(r2test(si, si+k)<r2_threshold) /*if ">=", keep growing on the right direction, but do not add si+k to inlist*/
-		  {
-		    for(j=0; j<npost; j++)
-		      {
-			newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
-			newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
-		      }
-		    mh_tmp=mantel(d); /*calculate the MH statistic*/
-		    if(mh_tmp>mh_threshold){
-		      mh_sum=mh_sum + mh_tmp; 
-		      inlist_length++;
-		      inlist[inlist_length-1]=si+k;
-		      newnhap=creatsubhap();}
-		    else flagr=0;
-		  }
-
-		/*if r2>r2_threshold, do NOT add si+k into inlist, but continue search;
-		  if r2<r2_threshold && mh_tmp>mh_threshold, add si+k into inlist;
-		  if r2<r2_threshold && mh_tmp<mh_threshold, do NOT add si+k into inlist,
-  		  but continue search*/
-
-		/*check si-k */
-		flagl=1;
-		if(r2test(si,si-k)<r2_threshold) /*if ">=", keep growing on the left direction, but do not add si-k to inlist*/
-		  {
-		    for(j=0; j<npost; j++)
-		      {
-			newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
-			newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
-		      }
-		    mh_tmp=mantel(d);
-		    if(mh_tmp>mh_threshold){
-		      mh_sum=mh_sum + mh_tmp;
-		      inlist_length++;
-		      inlist[inlist_length-1]=si-k;
-		      newnhap=creatsubhap();}
-		    else flagl=0;
-		  }
-
-		if(flagl!=1 && flagr!=1) flag=0; /* if either direction is kept for growing, stop*/
-	      }
-	    else
-	      {
-		/*check si-k*/
-		flagl=1;
-		if(r2test(si,si-k)<r2_threshold) 
-		  {
-		    for(j=0; j<npost; j++)
-		      {
-			newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
-			newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
-		      }
-		    mh_tmp=mantel(d);
-		    if(mh_tmp>mh_threshold){
-		      mh_sum=mh_sum + mh_tmp;
-		      inlist_length++;
-		      inlist[inlist_length-1]=si-k;
-		      newnhap=creatsubhap();}
-		    else flagl=0;
-		  }
-
-		/*check si+k */
-		flagr=1;
-		if(r2test(si, si+k)<r2_threshold) 
-		  {
-		    for(j=0; j<npost; j++)
-		      {
-			newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
-			newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
-		      }
-		    mh_tmp=mantel(d);
-		    if(mh_tmp>mh_threshold){
-		      mh_sum=mh_sum + mh_tmp;
-		      inlist_length++;
-		      inlist[inlist_length-1]=si+k;
-		      newnhap=creatsubhap();}
-		    else flagr=0;
-		  }
-
-		if(flagl!=1 && flagr!=1) flag=0;/*if neither direction is kept for growing, stop*/
-	      }
-
-	  if(si-k<0 && si+k<nsnp)
-	    {
-	      /*check si+k*/
-	      flagr=1;
-	      if(r2test(si, si+k)<r2_threshold) 
-		{
-		  for(j=0; j<npost; j++)
-		    {
-		      newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
-		      newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
-		    }
-		  mh_tmp=mantel(d);
-		  if(mh_tmp>mh_threshold){
-		    mh_sum=mh_sum + mh_tmp;
-		    inlist_length++;
-		    inlist[inlist_length-1]=si+k;
-		    newnhap=creatsubhap();}
-		  else flagr=0;
-		}
-	      if(flagr!=1) flag=0;
-	    }
-	  if(si+k>=nsnp && si-k>0)
-	    {
-	      /*check si-k*/
-	      flagl=1;
-	      if(r2test(si, si-k)<r2_threshold) 
-		{
-		  for(j=0; j<npost; j++)
-		    {
-		      newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
-		      newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
-		    }
-		  mh_tmp=mantel(d);
-		  if(mh_tmp>mh_threshold){
-		    mh_sum=mh_sum + mh_tmp;
-		    inlist_length++;
-		    inlist[inlist_length-1]=si-k;
-		    newnhap=creatsubhap();}
-		  else flagl=0;
-		}
-	      if(flagl!=1) flag=0;
-	    }
-	  return(mh_sum);
+  if( ((si - k) >= 0) && ((si + k) < nsnp)) {
+    if(pos[si] - pos[si-k] > pos[si+k] - pos[si]) {
+      /*check si+k*/
+      flagr=1;
+      if(r2test(si, si+k)<r2_threshold)  {
+	/*if ">=", keep growing on the right direction, but do not add si+k to inlist*/
+	
+	for(j=0; j<npost; j++) {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
+	}
+	mh_tmp=mantel(d); /*calculate the MH statistic*/
+	if(mh_tmp > mh_threshold) {
+	  mh_sum=mh_sum + mh_tmp; 
+	  inlist_length++;
+	  inlist[inlist_length-1]=si+k;
+	  newnhap=creatsubhap();
+	}
+	else flagr=0;
+      }
+      
+      /*if r2>r2_threshold, do NOT add si+k into inlist, but continue search;
+	if r2<r2_threshold && mh_tmp>mh_threshold, add si+k into inlist;
+	if r2<r2_threshold && mh_tmp<mh_threshold, do NOT add si+k into inlist,
+	but continue search*/
+      
+      /*check si-k */
+      flagl=1;
+      if(r2test(si,si-k)<r2_threshold) {
+	/*if ">=", keep growing on the left direction, but do not add si-k to inlist*/
+	for(j=0; j<npost; j++)  {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
+	}
+	mh_tmp=mantel(d);
+	if(mh_tmp > mh_threshold) {
+	  mh_sum=mh_sum + mh_tmp;
+	  inlist_length++;
+	  inlist[inlist_length-1]=si-k;
+	  newnhap=creatsubhap();
+	}
+	else flagl=0;
+      }
+      
+      if(flagl!=1 && flagr!=1) flag=0; /* if either direction is kept for growing, stop*/
+    }  else {
+      /*check si-k*/
+      flagl=1;
+      if(r2test(si,si-k)<r2_threshold) {
+	for(j=0; j<npost; j++) {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
+	}
+	mh_tmp=mantel(d);
+	if(mh_tmp>mh_threshold) {
+	  mh_sum=mh_sum + mh_tmp;
+	  inlist_length++;
+	  inlist[inlist_length-1]=si-k;
+	  newnhap=creatsubhap();
+	} else flagl=0;
+      }
+      
+      /*check si+k */
+      flagr=1;
+      if(r2test(si, si+k)<r2_threshold) {
+	for(j=0; j<npost; j++) {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
+	}
+	mh_tmp=mantel(d);
+	if(mh_tmp>mh_threshold){
+	  mh_sum=mh_sum + mh_tmp;
+	  inlist_length++;
+	  inlist[inlist_length-1]=si+k;
+	  newnhap=creatsubhap();
+	} else flagr=0;
+      }
+      
+      if(flagl!=1 && flagr!=1) flag=0;/*if neither direction is kept for growing, stop*/
+    }
+    
+    if(si-k<0 && si+k<nsnp) {
+      /*check si+k*/
+      flagr=1;
+      if(r2test(si, si+k)<r2_threshold) {
+	for(j=0; j<npost; j++) {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si+k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si+k];
+	}
+	mh_tmp=mantel(d);
+	if(mh_tmp>mh_threshold){
+	  mh_sum=mh_sum + mh_tmp;
+	  inlist_length++;
+	  inlist[inlist_length-1]=si+k;
+	  newnhap=creatsubhap();}
+	else flagr=0;
+      }
+      if(flagr!=1) flag=0;
+    }
+    if(si+k>=nsnp && si-k>0)  {
+      /*check si-k*/
+      flagl=1;
+      if(r2test(si, si-k)<r2_threshold) {
+	for(j=0; j<npost; j++)   {
+	  newhap1codesingle[j] = haplist[hap1code[j]-1][si-k];
+	  newhap2codesingle[j] = haplist[hap2code[j]-1][si-k];
+	}
+	mh_tmp=mantel(d);
+	if(mh_tmp>mh_threshold){
+	  mh_sum=mh_sum + mh_tmp;
+	  inlist_length++;
+	  inlist[inlist_length-1]=si-k;
+	  newnhap=creatsubhap();}
+	else flagl=0;
+      }
+      if(flagl!=1) flag=0;
+    }
+  }
+  return(mh_sum);
 }
 
+
 /*permute disease status*/
 static void permute(int *nondup_disease, double *perm_rand, int *perm_disease)
 {
@@ -451,7 +467,7 @@ static double r2test(int site1, int site2)
 /*calculate the Mantel-Haenszel statistic for */
 static double mantel(int *d)
 {
-  int i,j;
+  int i;
   double mh_stat=0, maxcount=0, num=0, denom=0, n00[newnhap], n0plus[newnhap], 
     n1plus[newnhap], nplus0[newnhap],nplus1[newnhap], ntotal[newnhap], n=0;
   /*initialize*/
@@ -534,15 +550,7 @@ static double mantel(int *d)
   if(denom==0) return(denom);
   mh_stat = pow(num,2)/denom;
   return(mh_stat);
-  /*
-    for(i=0; i<newnhap; i++)
-      {
-	printf("n00=%lf n0plus=%lf n1plus=%lf nplus0=%lf nplus1=%lf\n",
-	       n00[i], n0plus[i], n1plus[i], nplus0[i], nplus1[i]);
-      }
-	printf("num=%lf, denom=%lf, mh_stat=%lf\n", num, denom, mh_stat);
-	  printf("mh_stat\n%lf\n", mh_stat);
-  */
+
 }
 
 /*the chisq2h function calculates the chisquare statistic for a 2-by-h table */
@@ -550,7 +558,7 @@ static double chisq2h(int *d)
 {
   double n0[newnhap], n1[newnhap], n0sum=0, n1sum=0, nhapsum[newnhap];
   double en0[newnhap], en1[newnhap], ntotal=0, chi2h_stat=0;
-  int i, j;
+  int i;
 
   /*initialize*/
   for(i=0; i<newnhap; i++)
@@ -601,20 +609,13 @@ static double chisq2h(int *d)
       chi2h_stat = chi2h_stat + pow(trunn0[i]-en0[i],2)/en0[i] + 
 	pow(trunn1[i]-en1[i],2)/en1[i];
     }
-  /*
-  printf("k=%d\n", k);
-  printf("chisq2htest:\n");
-  for(i=0; i<ntrunhap; i++)
-    printf("n0=%lf n1=%lf en0=%lf en1=%lf %lf\n", trunn0[i], trunn1[i], en0[i], en1[i], trunhapsum[i]);
-  printf("n0sum=%lf n1sum=%lf ntotal=%lf\n", n0sum, n1sum, n0sum+n1sum);
-  printf("chi2h=%lf\n", chi2h_stat);
-  */
+  
   hap_df=ntrunhap-1;
   return(chi2h_stat);
 }
 
 /*the creatsubhap function identifies the subhaplotypes for a chosen set of loci*/
-static int creatsubhap()
+static int creatsubhap(void)
 {
   int i, j, a, duplicated[nhap];
   newnhap =0;
@@ -632,8 +633,8 @@ static int creatsubhap()
 	while(j<i && duplicated[i]==0)
 	  {
 	    for(a=0; a<inlist_length; a++){
-	      sprintf(&temp1[a], "%d", haplist[i][inlist[a]]);
-	      sprintf(&temp2[a], "%d", haplist[j][inlist[a]]);}
+	      snprintf(&temp1[a], inlist_length, "%d", haplist[i][inlist[a]]);
+	      snprintf(&temp2[a], inlist_length, "%d", haplist[j][inlist[a]]);}
 	    if(strncmp(temp1, temp2,inlist_length)==0) 
 	      {
 		duplicated[i] = 1;
@@ -654,17 +655,7 @@ static int creatsubhap()
       newhap1code[i] = newhapcode[hap1code[i]-1];
       newhap2code[i] = newhapcode[hap2code[i]-1];
     }
-
-  /*
-  printf("\n\nprintout from creatsub\n");
-  for(i=0; i<nhap; i++) printf("%d ", duplicated[i]); printf("newnhap=%d\n", newnhap);
-  for(i=0; i<nhap; i++) printf("%d,%d ", i+1,newhapcode[i]); printf("\n");
-  for(i=0; i<nhap; i++) printf("%d ", newhap1code[i]); printf("\n");
-  for(i=0; i<nhap; i++) printf("%d ", newhap2code[i]); printf("\n");
-  printf("createsub:\nnewnhap=%d\n",newnhap);*/
-
   return(newnhap);
-
 }
 
 /*the pchisq function calculates the cumulative distribution function of 
@@ -747,38 +738,31 @@ void gcf(double *gammcf, double a, double x, double *gln)
 #define ITMAX 100
 #define EPS 3.0e-7
 
-void gser (double *gamser, double a, double x, double *gln)
-{
+void gser (double *gamser, double a, double x, double *gln) {
 	double gammln(double xx);
-/*
-	void nrerror(char error_text[]);
-*/
+
 	int n;
 	double sum,del,ap;
 
 	*gln=gammln(a);
 	if (x <= 0.0) {
-/*
-		if (x < 0.0) nrerror("x less than 0 in routine gser");
-*/
-		*gamser=0.0;
-		return;
+	  
+	  *gamser=0.0;
+	  return;
 	} else {
-		ap=a;
-		del=sum=1.0/a;
-		for (n=1;n<=ITMAX;n++) {
-			++ap;
-			del *= x/ap;
-			sum += del;
-			if (fabs(del) < fabs(sum)*EPS) {
-				*gamser=sum*exp(-x+a*log(x)-(*gln));
-				return;
-			}
-		}
-/*
-		nrerror("a too large, ITMAX too small in routine gser");
-*/
-		return;
+	  ap=a;
+	  del=sum=1.0/a;
+	  for (n=1;n<=ITMAX;n++) {
+	    ++ap;
+	    del *= x/ap;
+	    sum += del;
+	    if (fabs(del) < fabs(sum)*EPS) {
+	      *gamser=sum*exp(-x+a*log(x)-(*gln));
+	      return;
+	    }
+	  }
+	  
+	  return;
 	}
 }
 #undef ITMAX
@@ -802,8 +786,7 @@ static int ranAS183_seed(int iseed1, int iseed2, int iseed3) {
 
 /***********************************************************************************/
 
-static double ranAS183()
-{
+static double ranAS183(void) {
    double u;
 
    ix = (171*ix) % 30269;
